'use strict';

var _class, _temp;

const Mservice = require('mservice');
const Mailer = require('ms-mailer-client');
const Errors = require('common-errors');
const merge = require('lodash/merge');
const fsort = require('redis-filtered-sort');
const { NotImplementedError } = Errors;
const defaultOpts = require('./defaults.js');

/**
 * @namespace Users
 */
module.exports = (_temp = _class = class Users extends Mservice {

  /**
   * @namespace Users
   * @param  {Object} opts
   * @return {Users}
   */
  constructor(opts = {}) {
    super(merge({}, Users.defaultOpts, opts));
    this.initAdminAccounts = require('./accounts/init-admin.js');
    this.initFakeAccounts = require('./accounts/init-dev.js');
    const config = this.config;

    const { error } = this.validateSync('config', config);
    if (error) {
      this.log.fatal('Invalid configuration:', error.toJSON());
      throw error;
    }

    this.on('plugin:connect:amqp', amqp => {
      this._mailer = new Mailer(amqp, config.mailer);
    });

    this.on('plugin:close:amqp', () => {
      this._mailer = null;
    });

    this.on('plugin:connect:redisCluster', redis => {
      fsort.attach(redis, 'fsort');
    });
  }

  /**
   * Getter for mailer client
   * @return {Object}
   */


  /**
   * Configuration options for the service
   * @type {Object}
   */
  get mailer() {
    const mailer = this._mailer;
    return mailer || this.emit('error', new NotImplementedError('amqp is not connected'));
  }

  /**
   * Getter for configuration
   * @return {Object}
   */
  get config() {
    return this._config;
  }

  /**
   * Initializes Admin accounts
   * @return {Promise}
   */


  /**
   * Initializes fake account for dev purposes
   * @return {Promise}
   */
}, _class.defaultOpts = defaultOpts, _temp);